import * as tf from '@tensorflow/tfjs-core';
import { MobileNetMultiplier } from './mobilenet';
import { Pose, PosenetInput } from './types';
export declare type PoseNetResolution = 161 | 193 | 257 | 289 | 321 | 353 | 385 | 417 | 449 | 481 | 513 | 801 | 1217;
export declare type PoseNetOutputStride = 32 | 16 | 8;
export declare type PoseNetArchitecture = 'ResNet50' | 'MobileNetV1';
export declare type PoseNetDecodingMethod = 'single-person' | 'multi-person';
export declare type PoseNetQuantBytes = 1 | 2 | 4;
export interface BaseModel {
    readonly inputResolution: PoseNetResolution;
    readonly outputStride: PoseNetOutputStride;
    predict(input: tf.Tensor3D): {
        [key: string]: tf.Tensor3D;
    };
    dispose(): void;
}
export interface ModelConfig {
    architecture: PoseNetArchitecture;
    outputStride: PoseNetOutputStride;
    inputResolution: PoseNetResolution;
    multiplier?: MobileNetMultiplier;
    modelUrl?: string;
    quantBytes?: PoseNetQuantBytes;
}
export interface InferenceConfig {
    flipHorizontal: boolean;
    decodingMethod: PoseNetDecodingMethod;
    maxDetections?: number;
    scoreThreshold?: number;
    nmsRadius?: number;
}
export declare const SINGLE_PERSON_INFERENCE_CONFIG: InferenceConfig;
export declare const MULTI_PERSON_INFERENCE_CONFIG: InferenceConfig;
export declare class PoseNet {
    baseModel: BaseModel;
    constructor(net: BaseModel);
    estimatePoses(input: PosenetInput, config?: InferenceConfig): Promise<Pose[]>;
    dispose(): void;
}
export declare function load(config?: ModelConfig): Promise<PoseNet>;
